/*****************************************************************************
 * Copyright (c) 1990-2000 Digi International Inc., All Rights Reserved
 * 
 * This software contains proprietary and confidential information of Digi
 * International Inc.  By accepting transfer of this copy, Recipient agrees
 * to retain this software in confidence, to prevent disclosure to others,
 * and to make no use of this software other than that for which it was
 * delivered.  This is an unpublished copyrighted work of Digi International
 * Inc.  Except as permitted by federal law, 17 USC 117, copying is strictly
 * prohibited.
 * 
 * Restricted Rights Legend
 *
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in sub-paragraph (c)(1)(ii) of The Rights in
 * Technical Data and Computer Software clause at DFARS 252.227-7031 or
 * subparagraphs (c)(1) and (2) of the Commercial Computer Software -
 * Restricted Rights at 48 CFR 52.227-19, as applicable.
 *
 * Digi International Inc. 11001 Bren Road East, Minnetonka, MN 55343
 *
 *****************************************************************************/

#ifndef PAM_SNMP_H
#define PAM_SNMP_H

static const char pam_snmp_h_rcsid[] = "@(#) " "$Id: pam_snmp.h,v 1.11 2000/04/19 22:45:47 chrism Exp $";

#include "tsquared.h"


BOOL UpdateAgentInfo(UCHAR * pResponseBuffer);

BOOL UpdateManagementShimVersion(UCHAR * pResponseBuffer, uint32 AdapterNumber);
BOOL UpdateManagementBoardCount(UCHAR * pResponseBuffer, uint32 AdapterNumber);
BOOL UpdateManagementBoardInfo(UCHAR * pResponseBuffer, uint32 AdapterNumber);
BOOL UpdateManagementDriverVersions(UCHAR * pResponseBuffer, uint32 AdapterNumber);

BOOL UpdateAdapterInfo(UCHAR * pResponseBuffer, uint32 AdapterNumber);
BOOL UpdateAdapterState(UCHAR * pResponseBuffer, uint32 AdapterNumber);
BOOL UpdateModemNationality(UCHAR * pResponseBuffer, uint32 AdapterNumber);
BOOL UpdateManagementProtocolVersion(UCHAR * pResponseBuffer, uint32 AdapterNumber);

BOOL UpdateDS1Configuration(UCHAR * pResponseBuffer, uint32 AdapterNumber, uint32 Port);
BOOL UpdateDS1ExtendedConfiguration(UCHAR * pResponseBuffer, uint32 AdapterNumber, uint32 Port);
BOOL UpdateDS1Status(UCHAR * pResponseBuffer, uint32 AdapterNumber, uint32 Port);
BOOL UpdateDS1ExtendedStatus(UCHAR * pResponseBuffer, uint32 AdapterNumber, uint32 Port);
BOOL UpdateDS1CT1SignalStatus(UCHAR * pResponseBuffer, uint32 AdapterNumber, uint32 Port, uint32 Channel);
BOOL UpdateDS1AllCT1SignalStatus(UCHAR * pResponseBuffer, uint32 AdapterNumber, uint32 Port);

BOOL UpdateBRIConfiguration(UCHAR * pResponseBuffer, uint32 AdapterNumber, uint32 Port);
BOOL UpdateBRIStatus(UCHAR * pResponseBuffer, uint32 AdapterNumber, uint32 Port);

BOOL UpdateSerialConfiguration(UCHAR * pResponseBuffer, uint32 AdapterNumber, uint32 Channel);
BOOL UpdateSerialStatus(UCHAR * pResponseBuffer, uint32 AdapterNumber, uint32 Channel);
BOOL UpdateSerialExtendedConfiguration(UCHAR * pResponseBuffer, uint32 AdapterNumber, uint32 Channel);
BOOL AlterSerialExtendedConfiguration(uint32 AdapterNumber, uint32 Channel, uchar * Values);

BOOL UpdateSerialExtendedStatus(UCHAR * pResponseBuffer, uint32 AdapterNumber, uint32 Channel);

BOOL UpdateCallStatus(UCHAR * pResponseBuffer, uint32 AdapterNumber, uint32 Port, uint32 Channel);
BOOL UpdateAllCallStatus(UCHAR * pResponseBuffer, uint32 AdapterNumber, uint32 Port);

BOOL UpdateModemConfiguration(UCHAR * pResponseBuffer, uint32 AdapterNumber, uint32 Channel);
BOOL AlterModemConfiguration(uint32 AdapterNumber, uint32 Channel, uchar * Values);
BOOL UpdateModemStatus(UCHAR * pResponseBuffer, uint32 AdapterNumber, uint32 Channel);
BOOL UpdateModemExtendedConfiguration(UCHAR * pResponseBuffer, uint32 AdapterNumber, uint32 Channel);
BOOL UpdateModemExtendedStatus(UCHAR * pResponseBuffer, uint32 AdapterNumber, uint32 Channel);

BOOL UpdateDChannelExtendedConfiguration(UCHAR * pResponseBuffer, uint32 AdapterNumber, uint32 Port);
BOOL UpdateDChannelExtendedStatus(UCHAR * pResponseBuffer, uint32 AdapterNumber, uint32 Port);

BOOL UpdateBChannelStatus(UCHAR * pResponseBuffer, uint32 AdapterNumber, uint32 Port);
BOOL UpdateBChannelExtendedConfiguration(UCHAR * pResponseBuffer, uint32 AdapterNumber, uint32 Port);
BOOL AlterBChannelExtendedConfiguration(uint32 AdapterNumber, uint32 Channel, uchar * Values);
BOOL UpdateBChannelExtendedStatus(UCHAR * pResponseBuffer, uint32 AdapterNumber, uint32 Port);

BOOL AlterModemExtendedConfiguration(uint32 AdapterNumber, uint32 Channel, uchar * Values);

BOOL UpdateSystemStatus(UCHAR * pResponseBuffer);
BOOL UpdateAllChannelInfos(UCHAR * pResponseBuffer, uint32 AdapterNumber, BOOL getextradata);
BOOL UpdateAllSerialSummaries(UCHAR * pResponseBuffer, uint32 AdapterNumber);
BOOL UpdateAllSerialDetails(UCHAR * pResponseBuffer, uint32 AdapterNumber);

BOOL UpdateSerialTestChannelMode(UCHAR * pResponseBuffer, uint32 AdapterNumber, uint32 Channel);
BOOL AlterSerialTestChannelMode(uint32 AdapterNumber, uint32 Channel, uchar * Values);
BOOL AlterSerialTestStatus(uint32 AdapterNumber, uint32 Channel, uchar * Values);
BOOL AlterSerialTestConfiguration(uint32 AdapterNumber, uint32 Channel, uchar * Values);


#define PAM_AgentInfo									0
#define PAM_ManagementShimVersion					1
#define PAM_ManagementBoardCount						2
#define PAM_ManagementBoardInfo						3
#define PAM_ManagementDriverVersions				4
#define PAM_AdapterInfo									5
#define PAM_AdapterState								6
#define PAM_ModemNationality							7
#define PAM_ManagementProtocolVersion				8
#define PAM_DS1Configuration							9
#define PAM_DS1ExtendedConfiguration				10
#define PAM_DS1Status									11
#define PAM_DS1ExtendedStatus							12
#define PAM_DS1CT1SignalStatus						13
#define PAM_DS1AllCT1SignalStatus					14
#define PAM_BRIConfiguration							15
#define PAM_BRIStatus									16
#define PAM_SerialConfiguration						17
#define PAM_SerialStatus								18
#define PAM_SerialExtendedConfiguration			19
#define PAM_SerialExtendedStatus						20
#define PAM_CallStatus									21
#define PAM_AllCallStatus								22
#define PAM_ModemConfiguration						23
#define PAM_ModemStatus									24
#define PAM_ModemExtendedConfiguration				25
#define PAM_ModemExtendedStatus						26
#define PAM_DChannelExtendedConfiguration			27
#define PAM_DChannelExtendedStatus					28
#define PAM_BChannelStatus								29
#define PAM_BChannelExtendedConfiguration			30
#define PAM_BChannelExtendedStatus					31
#define PAM_SerialTestChannelMode					32
#define PAM_SerialTestStatus							33
#define PAM_SerialTestConfiguration					34
#define PAM_SystemStatus								100
#define PAM_AllChannelInfos							101
#define PAM_AllChannelInfosExt						102
#define PAM_AllSerialSummaries						103
#define PAM_AllSerialDetails							104

#endif // PAM_SNMP_H
